from django import forms
from edc_form_validators import FormValidator, FormValidatorMixin

from edc_adherence.form_validator_mixin import MedicationAdherenceFormValidatorMixin
from edc_adherence.model_form_mixin import MedicationAdherenceFormMixin

from .models import MedicationAdherence


class MedicationAdherenceFormValidator(MedicationAdherenceFormValidatorMixin, FormValidator):
    pass


class MedicationAdherenceForm(
    MedicationAdherenceFormMixin, FormValidatorMixin, forms.ModelForm
):

    form_validator_cls = MedicationAdherenceFormValidator

    class Meta:
        model = MedicationAdherence
        fields = "__all__"



from django.test import TestCase
from edc_appointment.models import Appointment
from edc_appointment.tests.helper import Helper
from edc_constants.constants import NEVER, OTHER, YES
from edc_facility import import_holidays
from edc_list_data import site_list_data
from edc_reference import site_reference_configs
from edc_visit_schedule import site_visit_schedules

from edc_adherence import list_data
from edc_adherence.models import NonAdherenceReasons
from edc_adherence.tests.admin import MedicationAdherenceAdmin, my_admin_site

from ..forms import MedicationAdherenceForm
from ..models import MedicationAdherence, SubjectVisit
from ..visit_schedules import visit_schedule


class TestAdherence(TestCase):
    @classmethod
    def setUpTestData(cls):
        import_holidays()

    def setUp(self) -> None:
        site_list_data.initialize()
        site_list_data.register(list_data, app_name="edc_adherence")
        site_list_data.load_data()
        site_visit_schedules._registry = {}
        site_visit_schedules.register(visit_schedule)
        site_reference_configs.register_from_visit_schedule(
            visit_models={"edc_appointment.appointment": "edc_adherence.subjectvisit"}
        )
        self.subject_identifier = "1234"
        self.helper = Helper(subject_identifier=self.subject_identifier)
        self.helper.consent_and_put_on_schedule(
            visit_schedule_name="visit_schedule", schedule_name="schedule"
        )
        appointments = Appointment.objects.filter(subject_identifier=self.subject_identifier)
        self.subject_visit = SubjectVisit.objects.create(appointment=appointments[0])

    def test_ok(self):

        opts = dict(
            visual_score_slider=90,
            visual_score_confirmed=90,
            last_missed_pill=NEVER,
            pill_count_performed=YES,
            pill_count=30,
            other_missed_pill_reason=None,
        )

        obj = MedicationAdherence(subject_visit=self.subject_visit, **opts)
        obj.save()
        obj.delete()

        opts = dict(
            visual_score_slider=90,
            visual_score_confirmed=90,
            last_missed_pill="today",
            pill_count_performed=YES,
            pill_count=20,
            other_missed_pill_reason=None,
        )
        obj = MedicationAdherence(subject_visit=self.subject_visit, **opts)
        obj.save()
        obj.missed_pill_reason.add(NonAdherenceReasons.objects.get(name="make_me_ill"))

    def test_form(self):
        data = dict(
            visual_score_slider=90,
            visual_score_confirmed=90,
            last_missed_pill=NEVER,
            pill_count_performed=YES,
            pill_count=30,
            other_missed_pill_reason=None,
            subject_visit=self.subject_visit,
            report_datetime=self.subject_visit.report_datetime,
        )
        form = MedicationAdherenceForm(data=data)
        form.is_valid()
        self.assertDictEqual(form._errors, {})

        data.update(visual_score_confirmed=80)
        form = MedicationAdherenceForm(data=data)
        form.is_valid()
        self.assertIn("visual_score_confirmed", form._errors)
        data.update(visual_score_confirmed=90)

        data.update(last_missed_pill="today")
        form = MedicationAdherenceForm(data=data)
        form.is_valid()
        self.assertIn("missed_pill_reason", form._errors)

        data.update(missed_pill_reason=NonAdherenceReasons.objects.filter(name=OTHER))
        form = MedicationAdherenceForm(data=data)
        form.is_valid()
        self.assertIn("other_missed_pill_reason", form._errors)

        data.update(
            other_missed_pill_reason="i feel the pull on the rope, let me off at the rainbow"
        )
        form = MedicationAdherenceForm(data=data)
        form.is_valid()
        self.assertDictEqual(form._errors, {})

    def test_admin(self):
        my_admin_site.register(MedicationAdherence, MedicationAdherenceAdmin)
        MedicationAdherenceAdmin(MedicationAdherence, my_admin_site)



from dateutil.relativedelta import relativedelta
from edc_visit_schedule import Crf, FormsCollection, Schedule, Visit, VisitSchedule

crfs = FormsCollection(
    Crf(show_order=1, model="edc_adherence.medicationadherence", required=True)
)

visit = Visit(
    code="1000",
    timepoint=0,
    rbase=relativedelta(days=0),
    rlower=relativedelta(days=0),
    rupper=relativedelta(days=6),
    requisitions=None,
    crfs=crfs,
    requisitions_unscheduled=None,
    crfs_unscheduled=None,
    allow_unscheduled=False,
    facility_name="5-day-clinic",
)


schedule = Schedule(
    name="schedule",
    onschedule_model="edc_adherence.onschedule",
    offschedule_model="edc_adherence.offschedule",
    appointment_model="edc_appointment.appointment",
    consent_model="edc_adherence.subjectconsent",
)

visit_schedule = VisitSchedule(
    name="visit_schedule",
    offstudy_model="edc_adherence.offstudy",
    death_report_model="edc_adherence.deathreport",
    locator_model="edc_adherence.locator",
)

schedule.add_visit(visit)

visit_schedule.add_schedule(schedule)



from django.db import models
from django.db.models import PROTECT
from edc_appointment.models import Appointment
from edc_crf.crf_model_mixin import CrfModelMixin
from edc_metadata.model_mixins.creates import CreatesMetadataModelMixin
from edc_model import models as edc_models
from edc_model.models import BaseUuidModel
from edc_reference.model_mixins import ReferenceModelMixin
from edc_registration.model_mixins import UpdatesOrCreatesRegistrationModelMixin
from edc_sites.models import SiteModelMixin
from edc_utils import get_utcnow
from edc_visit_tracking.constants import SCHEDULED
from edc_visit_tracking.model_mixins import VisitModelMixin

from edc_adherence.model_mixins import MedicationAdherenceModelMixin


class DeathReport(edc_models.BaseUuidModel):
    subject_identifier = models.CharField(max_length=50)


class OffStudy(edc_models.BaseUuidModel):
    subject_identifier = models.CharField(max_length=50)

    offstudy_datetime = models.DateTimeField(default=get_utcnow)


class Locator(edc_models.BaseUuidModel):
    subject_identifier = models.CharField(max_length=50)


class SubjectConsent(UpdatesOrCreatesRegistrationModelMixin, edc_models.BaseUuidModel):
    subject_identifier = models.CharField(max_length=50)

    consent_datetime = models.DateTimeField()


class OnSchedule(edc_models.BaseUuidModel):
    subject_identifier = models.CharField(max_length=50)

    onschedule_datetime = models.DateTimeField(default=get_utcnow)


class OffSchedule(edc_models.BaseUuidModel):
    subject_identifier = models.CharField(max_length=50)

    offschedule_datetime = models.DateTimeField(default=get_utcnow)


class SubjectVisit(
    VisitModelMixin,
    ReferenceModelMixin,
    CreatesMetadataModelMixin,
    SiteModelMixin,
    BaseUuidModel,
):

    appointment = models.OneToOneField(
        Appointment, on_delete=PROTECT, related_name="edc_adherence_appointment"
    )

    subject_identifier = models.CharField(max_length=50)

    reason = models.CharField(max_length=25, default=SCHEDULED)


class MedicationAdherence(
    MedicationAdherenceModelMixin, CrfModelMixin, edc_models.BaseUuidModel
):
    class Meta(CrfModelMixin.Meta, edc_models.BaseUuidModel.Meta):
        verbose_name = "Medication Adherence"
        verbose_name_plural = "Medication Adherence"



from django.contrib import admin
from django.contrib.admin import AdminSite as DjangoAdminSite
from edc_model_admin import SimpleHistoryAdmin
from edc_model_admin.dashboard import ModelAdminCrfDashboardMixin

from edc_adherence.model_admin_mixin import MedicationAdherenceAdminMixin

from .forms import MedicationAdherenceForm
from .models import MedicationAdherence


class CrfModelAdmin(ModelAdminCrfDashboardMixin, SimpleHistoryAdmin):

    pass


class AdminSite(DjangoAdminSite):
    pass


my_admin_site = AdminSite(name="my_admin_site")


@admin.register(MedicationAdherence)
class MedicationAdherenceAdmin(MedicationAdherenceAdminMixin, CrfModelAdmin):

    form = MedicationAdherenceForm
